
C                           Subroutine AB_LU4FRM
C
C   The following subroutine is a modification  of the generic interface
C   LU4FRM.FOR (by T. Sjostrand) for a 4-fermion code to Jetset.
C   It is intended for use in connection with WPHACT.

C...An electroweak generator is supposed to
C...have produced two fermions, two antifermions and an arbitrary
C...number of photons. These particles are stored in the HEPEVT
C...common block. The allowed order is specified by a standard.
C...In brief, the final fermions should appear in the order
C...fermion (1) - antifermion (2) - fermion (3) - antifermion (4).
C...The flavour pairs should be arranged so that, if possible, the
C...first two could come from a W+ and the second two from a W-;
C...else each pair should have flavours consistent with a Z0.
C...The subroutine LU4FRM is supposed to read the configuration,
C...and call JETSET to do parton showers and fragmentation.

C   The colour flow need not be unique.
C   WPHACT however generates events in which the color structure is well
C   defined (with probability corresponding to the relative contribution if
C   both structures are)
C   and indicated by the integer input ichar:
C   ICHAR=0 corresponds to the color structure characteristic of neutral
C           currents (NC) with no identical particles in the final state,
C           i.e. (1) (2) and (3) (4) are color singlets.
C   ICHAR=1 corresponds to the color structure characteristic of charged
C           currents (CC), in which (1) (4) and (2) (3) are color singlets.
C   When both color structures are possible, one is chosen with probability
C   proportional to its contribution to the cross section. A flag
C   of WPHACT determines wether the interference is added to CC or NC.

C...Final-state QED radiation may be allowed or inhibited:
C...IRAD = 0 : no final-state photon radiation.
C...     = 1 : photon radiation inside each final fermion pair.
C...
C...tau lepton decay may be  handled by QCD generator or not.
C...ITAU = 0 : taus are considered stable by QCD generator.
C...     = 1 : taus are allowed to decay by QCD generator.
C...
C...IERR is an error flag, used both as input and output.
C...At input, 0 means leave routine in case of error,
C...   nonzero means stop program execution.
C...At output, 0 means acceptable fermion configuration,
C...   nonzero means treatment aborted for some reason.
C...It is up to the writer of the main program to pick error strategy,
C...i.e. to let the program crash or try to fix errors.


      SUBROUTINE AB_LU4FRM(ichar,IRAD,ITAU,IERR)

      COMMON/LUJETS/N,K(4000,5),P(4000,5),V(4000,5)
      COMMON/LUDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      DIMENSION IJOIN(2)

C...Call LUHEPC to convert from HEPEVT to LUJETS common.
      INERR=0
      MSTU(28)=0
      CALL LUHEPC(2)
      IF(MSTU(28).EQ.8) INERR=1

C...Loop through entries and pick up all final fermions/antifermions.
      I1=0
      I2=0
      I3=0
      I4=0
      DO 100 I=1,N
        IF(K(I,1).LE.0.OR.K(I,1).GT.10) GOTO 100
        KFA=IABS(K(I,2))
        IF((KFA.GE.1.AND.KFA.LE.6).OR.(KFA.GE.11.AND.KFA.LE.16)) THEN
          IF(K(I,2).GT.0) THEN
            IF(I1.EQ.0) THEN
              I1=I
            ELSEIF(I3.EQ.0) THEN
              I3=I
            ELSE
              INERR=2
              CALL LUERRM(6,'(LU4FRM:) more than two fermions')
            ENDIF
          ELSE
            IF(I2.EQ.0) THEN
              I2=I
            ELSEIF(I4.EQ.0) THEN
              I4=I
            ELSE
              INERR=3
              CALL LUERRM(6,'(LU4FRM:) more than two antifermions')
            ENDIF
          ENDIF
        ENDIF
  100 CONTINUE

C...Check that event is arranged according to conventions.
      IF(I1.EQ.0.OR.I2.EQ.0.OR.I3.EQ.0.OR.I4.EQ.0) THEN
        INERR=4
        CALL LUERRM(6,'(LU4FRM:) event contains too few fermions')
      ENDIF
      IF(I2.LT.I1.OR.I3.LT.I2.OR.I4.LT.I3) THEN
        INERR=5
        CALL LUERRM(6,'(LU4FRM:) fermions arranged in wrong order')
      ENDIF

C...Check which fermion pairs are quarks and which leptons.
      IF(IABS(K(I1,2)).LT.10.AND.IABS(K(I2,2)).LT.10) THEN
        IQL12=1
      ELSEIF(IABS(K(I1,2)).GT.10.AND.IABS(K(I2,2)).GT.10) THEN
        IQL12=2
      ELSE
        INERR=6
        CALL LUERRM(6,'(LU4FRM:) first fermion pair inconsistent')
      ENDIF
      IF(IABS(K(I3,2)).LT.10.AND.IABS(K(I4,2)).LT.10) THEN
        IQL34=1
      ELSEIF(IABS(K(I3,2)).GT.10.AND.IABS(K(I4,2)).GT.10) THEN
        IQL34=2
      ELSE
        INERR=7
        CALL LUERRM(6,'(LU4FRM:) second fermion pair inconsistent')
      ENDIF

C...Return or stop program in case of problems.
      IF(INERR.EQ.0) THEN
        IERR=0
      ELSEIF(IERR.EQ.0) THEN
        IERR=INERR
        RETURN
      ELSE
        WRITE(6,*) ' ERROR: listing of faulty event follows:'
        CALL LULIST(2)
        WRITE(6,*) ' Fermions found in lines ',I1,I2,I3,I4
        WRITE(6,*) ' Error type in event above is ',INERR
        WRITE(6,*) ' Program execution will be stopped now'
        WRITE(6,*) ' since main program does not correct errors!'
        STOP
      ENDIF

C...Decide whether to allow or not photon radiation in showers.
      MSTJ(41)=2
      IF(IRAD.EQ.0) MSTJ(41)=1

C...Decide on colour pairing.
      IF(IQL12.EQ.2.AND.IQL34.EQ.2) THEN
        NPAIR=0
      ELSEIF(IQL12.EQ.1.AND.IQL34.EQ.2) THEN
        NPAIR=1
        IP1=I1
        IP2=I2
      ELSEIF(IQL12.EQ.2.AND.IQL34.EQ.1) THEN
        NPAIR=1
        IP1=I3
        IP2=I4
      ELSE
        NPAIR=2
        IP1=I1
        IP3=I3
        IF(ichar.EQ.0) THEN
          IP2=I2
          IP4=I4
        ELSE
          IP2=I4
          IP4=I2
        ENDIF
      ENDIF

C...Do colour joining and parton showers.
      IF(NPAIR.GE.1) THEN
        IJOIN(1)=IP1
        IJOIN(2)=IP2
        CALL LUJOIN(2,IJOIN)
        PM12S=(P(IP1,4)+P(IP2,4))**2-(P(IP1,1)+P(IP2,1))**2-
     &    (P(IP1,2)+P(IP2,2))**2-(P(IP1,3)+P(IP2,3))**2
        CALL LUSHOW(IP1,IP2,SQRT(MAX(0.,PM12S)))
      ENDIF
      IF(NPAIR.EQ.2) THEN
        IJOIN(1)=IP3
        IJOIN(2)=IP4
        CALL LUJOIN(2,IJOIN)
        PM34S=(P(IP3,4)+P(IP4,4))**2-(P(IP3,1)+P(IP4,1))**2-
     &    (P(IP3,2)+P(IP4,2))**2-(P(IP3,3)+P(IP4,3))**2
        CALL LUSHOW(IP3,IP4,SQRT(MAX(0.,PM34S)))
      ENDIF

C...Do fragmentation and decays. Possibly except tau decay.
      IF(ITAU.EQ.0) THEN
        IF(IABS(K(I1,2)).EQ.15) K(I1,1)=11
        IF(IABS(K(I2,2)).EQ.15) K(I2,1)=11
        IF(IABS(K(I3,2)).EQ.15) K(I3,1)=11
        IF(IABS(K(I4,2)).EQ.15) K(I4,1)=11
      ENDIF
      CALL LUEXEC
      IF(ITAU.EQ.0) THEN
        IF(IABS(K(I1,2)).EQ.15) K(I1,1)=1
        IF(IABS(K(I2,2)).EQ.15) K(I2,1)=1
        IF(IABS(K(I3,2)).EQ.15) K(I3,1)=1
        IF(IABS(K(I4,2)).EQ.15) K(I4,1)=1
      ENDIF

      END
